﻿using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;
using Autofac;
using FitConnect.Security;
using FitConnect.Services;
using FitConnect.Services.Models;
using Moq;
using Newtonsoft.Json;

namespace MockContainer;

public static class Container {
    public static X509Certificate2 GetCertificate() => RsaEncryption.CreateSelfSignedCertificate();

    public static IContainer Create() {
        var builder = new ContainerBuilder();

        var routeService = new Mock<IRouteService>();
        routeService.Setup(r => r.GetDestinationIdAsync(
                It.Is<string>(s => !string.IsNullOrWhiteSpace(s)),
                It.IsAny<string>(),
                It.IsAny<string>(),
                It.IsAny<string?>()))
            .Returns(() => Task.Run(() => "destination"));


        var oAuthService = new Mock<IOAuthService>();
        oAuthService.Setup(r => r.AuthenticateAsync(
                It.Is<string>(s => !string.IsNullOrWhiteSpace(s)),
                It.Is<string>(s => !string.IsNullOrWhiteSpace(s)),
                It.IsAny<string?>()
            ))
            .Returns((string cId, string cSecret, string scp) => {
                if (cId == "clientId" && cSecret == "clientSecret")
                    return Task.Run<OAuthAccessToken?>(() =>
                        new OAuthAccessToken {
                            ExpiresIn = 1800,
                            AccessToken = "accessToken",
                            Scope = scp,
                            TokenType = "Bearer"
                        });

                throw new InvalidCredentialException(
                    "Invalid credentials: \"error\": \"invalid_client\"");
            });

        var submissionService = new Mock<ISubmissionService>();
        submissionService.Setup(s =>
            s.ListSubmissions(It.IsAny<string>(), It.IsAny<int>(), It.IsAny<int>())).Returns(
            () =>
                Task.Run(() => JsonConvert.DeserializeObject<SubmissionsForPickupDto>(
                    @"{""offset"":0,""count"":3,""totalCount"":3,""submissions"":[{""destinationId"":""879ee109-a690-4db8-ab32-424284184d7d"",""submissionId"":""ce75a6b8-d72f-4b94-b09e-af6be35bc2ae""},{""destinationId"":""19c8489b-29b8-422f-b7db-919852cfb04b"",""submissionId"":""e364430f-5a3b-4284-ba9a-f2867ba421e6""},{""destinationId"":""80a0aac3-148d-42bb-9366-516ce6355348"",""submissionId"":""530ba588-2db9-4899-ab0d-0c0b57689271""}]}"))
        );

        builder.Register(c => oAuthService.Object).As<IOAuthService>();
        builder.Register(c => routeService.Object).As<IRouteService>();
        builder.Register(c => submissionService.Object).As<ISubmissionService>();
        return builder.Build();
    }
}
